unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, TAGraph, TATools, TASeries, TASources, Forms,
  Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, TACustomSource,
  TAChartListbox, Types, TALegend, TACustomSeries;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Chart1: TChart;
    Chart1LineSeries1: TLineSeries;
    Chart1LineSeries2: TLineSeries;
    Chart1LineSeries3: TLineSeries;
    ChartToolset1: TChartToolset;
    Label1: TLabel;
    LineExtenderTool: TDataPointDragTool;
    CurveMoverTool: TDataPointHintTool;
    ChartListbox1: TChartListbox;
    Panel1: TPanel;
    procedure Button1Click(Sender: TObject);
    procedure ChartListbox1AddSeries(ASender: TChartListbox;
      ASeries: TCustomChartSeries; AItems: TChartLegendItems; var ASkip: Boolean
      );
    procedure ChartListbox1ItemClick(ASender: TObject; AIndex: Integer);
    procedure CurveMoverToolAfterMouseDown(ATool: TChartTool; APoint: TPoint);
    procedure CurveMoverToolAfterMouseMove(ATool: TChartTool; APoint: TPoint);
    procedure FormCreate(Sender: TObject);
    procedure LineExtenderToolAfterMouseDown(ATool: TChartTool; APoint: TPoint);
    procedure LineExtenderToolAfterMouseMove(ATool: TChartTool; APoint: TPoint);
  private
    { private declarations }
    FMouseDownPt: TPoint;
    FSlope: Double;
  public
    { public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

uses
  LCLIntf, TAChartUtils;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
const
  XMIN = 0;
  XMAX = 100;
  N = 100;
var
  i: Integer;
  x,y: Double;
begin
  FMouseDownPt.X := MaxInt;
  FMouseDownPt.Y := MaxInt;

  // 1.Kurve (Gerade)
  for i:=0 to 1 do
  begin
    x := i * XMAX + XMIN;
    y := x;
    Chart1LineSeries1.AddXY(x, y);
  end;

  // 2.Kurve (Parabel)
  for i:=0 to N-1 do
  begin
    x := i/(N-1) * XMAX + XMIN;
    y := x*x*1e-2;
    Chart1LineSeries2.AddXY(x, y);
  end;

  // 3.Kurve (flachere Parabel)
  for i:=0 to N-1 do begin
    x := i / (N-1) * XMAX + XMIN;
    y := 0.5 * x*x*1e-2;
    Chart1LineSeries3.AddXY(x, y);
  end;

  // Extent einfrieren, sonst verändert sich die Sklaierung beim Verschieben
  Chart1.Prepare;
  Chart1.Extent.FixTo(Chart1.GetFullExtent);
end;

procedure TForm1.LineExtenderToolAfterMouseDown(ATool: TChartTool;
  APoint: TPoint);
begin
  if TDataPointTool(ATool).Series <> Chart1LineSeries1 then
    exit;
  with Chart1LineSeries1 do
    FSlope := (YValue[1] - YValue[0]) / (XValue[1] - XValue[0]);
end;

procedure TForm1.LineExtenderToolAfterMouseMove(ATool: TChartTool;
  APoint: TPoint);
var
  x: Double;
begin
  x := Chart1.XImageToGraph(APoint.X);
  with Chart1LineSeries1 do
  begin
    XValue[1] := XValue[0] + FSlope * (x - XValue[0]);
//    XValue[1] := x;
  end;
end;

procedure TForm1.ChartListbox1ItemClick(ASender: TObject; AIndex: Integer);
var
  ser: TCustomChartSeries;
  i: Integer;
begin
  CurveMoverTool.Enabled := ChartListbox1.ItemIndex > -1;
  if CurveMoverTool.Enabled then begin
    ser := ChartListbox1.Series[ChartListbox1.ItemIndex];
    if ser = Chart1LineSeries1 then
      CurveMovertool.Enabled := false
    else
      CurveMoverTool.AffectedSeries := IntToStr(ser.Index);

    for i:=0 to ChartListbox1.Items.Count-1 do
      if i = ChartListbox1.ItemIndex then
        (ChartListbox1.Series[i] as TLineSeries).LinePen.Width := 3
      else
        (ChartListbox1.Series[i] as TLineSeries).LinePen.Width := 1;
  end;
end;

procedure TForm1.ChartListbox1AddSeries(ASender: TChartListbox;
  ASeries: TCustomChartSeries; AItems: TChartLegendItems; var ASkip: Boolean);
begin
  // Die Gerade soll nicht in der Listbox angezeigt werden.
  ASkip := ASeries = Chart1LineSeries1;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  ser: TLineSeries;
  i: Integer;
begin
  ser := TLineSeries.Create(Chart1);
  ser.SeriesColor := rgb(Random(256), Random(256), Random(256));
  ser.Title := 'Kurve Nr. ' + IntToStr(Chart1.SeriesCount);
  for i:=0 to 100 do
    ser.AddXY(i, (Random - 0.5)*(20 + Random(80)) + 50);
  Chart1.AddSeries(ser);
end;

procedure TForm1.CurveMoverToolAfterMouseDown(ATool: TChartTool; APoint: TPoint);
begin
  FMouseDownPt := APoint;
end;

procedure TForm1.CurveMoverToolAfterMouseMove(ATool: TChartTool; APoint: TPoint);
var
  i: Integer;
  ser: TChartSeries;
  dx: Double;
begin
  ser := TDataPointTool(ATool).Series as TChartSeries;
  if ser = nil then
    exit;

  // Verschobene Distanz in Welt-Koordinaten umrechnen
  dx := Chart1.XImageTograph(APoint.X) - Chart1.XImageToGraph(FMouseDownPt.X);

  // und zu den Datenpunkten hinzuaddieren
  ser.ListSource.BeginUpdate;
  try
    for i:=0 to ser.Count-1 do
      ser.XValue[i] := ser.XValue[i] + dx;
  finally
    ser.ListSource.EndUpdate;
  end;
  FMouseDownPt := APoint;
end;

end.

